function [sys,x0,str,ts] = fcc_model(t,x,u,flag)
% fcc_model is an S-function which implement a simplified model of FCC unit
% (simplified Kurihara model)
% Rf:
% [1] Mc Farlane & Bacon (Queen's University)
% [2] Lee & Kugelman (MOBIL Research)
%
% Dispatch the flag.
%
switch flag,
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes; % Initialization
  case 1
    sys = mdlDerivatives(t,x,u); % Calculate derivatives
  case 3
    sys = mdlOutputs(t,x,u); % Calculate outputs  
  case { 2, 4, 9 } % Unused flags
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); % Error handling
end
% End of csfunc.
%==============================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the 
% S-function.
%==============================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes()
%
% Call simsizes for a sizes structure, fill it in and convert it 
% to a sizes array.
%
sizes = simsizes;
sizes.NumContStates  = 4;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 7;
sizes.NumInputs      = 2;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);
%
% Initialize the initial conditions.
%
Tra = 775;
Trg = 943;
Crc = 0.3;
Csc = 1.2;
x0 = [Tra;Trg;Crc;Csc];
%
% str is an empty matrix.
%
str = [];
%
% Initialize the array of sample times; in this example the sample 
% time is continuous, so set ts to 0 and its offset to 0.
%
ts = [0 0];
% End of mdlInitializeSizes.
%
%==============================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%==============================================================
function sys = mdlDerivatives(t,x,u)
% NAMES / DESCRIPTION / UNITS
%   R     Ideal gas constant J/mol.K
%   H     Holdup kg
%   R     Rate kg/s
%   S     Specific heat J/kg.K
%   P     Pressure kPa
%   T     Temperature K
%   D     Heat J/kg
%   A     Activation energy J/mol
%   F     factor, without unit
%   k     empirical constant rates /kPa.s
R = 8.314;
%
% Reactor
% -------
Hra = 1.85e4;
Pra = 211.7;
DHcr = 4.65e5;
% feed
Rtf = 41;
Ttf = 492.8;
Stf = 3140;
% Reaction 1
Acr = 6.28e4;
kcr = 8.31e-2;
% Reaction 2
Acc = 4.18e4;
kcc = 2.66e-4;
%
%Fcf = 5.6e-4;
% Regenerator
% -----------
Hrg = 1.53e5;
Prg = 254.4;
DHfv = 1.74e5;
DHrg = 3.02e7;
% Air
Rai = 26;
Tai = 394;
Sai = 1130;
% Catalyst
Rc  = 290;
Sc  = 1047;
% Reaction (02)
Aor = 1.47e5;
Tra=x(1);
Trg=x(2);
Crc=x(3);
Csc=x(4);
Rai=u(1);
Fcf=u(2);

Ccat=sqrt(100*Pra*Hra*kcc*exp( -Acc/(R*Tra) ) / (Rc*Crc^0.06));

Kcr = kcr/(Ccat*Crc^0.15)*exp( -Acr/(R*Tra) );
Rcr = Kcr*Pra*Hra;
Ctf = 1 / ( 1 + Rtf/Rcr);
dTra = ( Sc*Rc*(Trg-Tra) + Stf*Rtf*(Ttf-Tra) - Rtf*DHfv - Rtf*Ctf*DHcr ) / (Sc*Hra);

Kor = 1.16e-5*exp( Aor/R*( 1/866.7 - 1/Trg ) );
Kod = 6.34e-9*Rai^2;
Ofg = 21*exp( - (Prg*Hrg/Rai) / ( 1/Kod + 1/(Kor*Crc) ) );
Rcb = Rai*(21-Ofg)/242;
dTrg = ( Sc*Rc*(Tra-Trg) + Sai*Rai*(Tai-Trg) + Rcb*DHrg ) / (Sc*Hrg);

dCrc = ( Rc*(Csc-Crc) - 100*Rcb )/Hrg;

Kcc = kcc/(Ccat*Crc^0.06)*exp( -Acc/(R*Tra) );
Rcc = Kcc*Pra*Hra;
Rad = Fcf*Rtf;
Rcf = Rcc + Rad;
dCsc = ( Rc*(Crc-Csc) + 100*Rcf )/Hra;

if t==0,
    save Actual_x0.mat Tra Trg Crc Csc Fcf Rcb
    disp('Actual initial state saved as Tra Trg Crc Csc Fcf Rcb in Actual_x0.mat'),
end,

sys = 3600*[dTra;dTrg;dCrc;dCsc];
% End of mdlDerivatives.
%
%==============================================================
% mdlOutputs
% Return the block outputs.
%==============================================================
%
function sys = mdlOutputs(t,x,u)
R = 8.314;
Hrg = 1.53e5;
Prg = 254.4;
Aor = 1.47e5;
Trg=x(2);
Crc=x(3);
Rai=u(1);
Fcf=u(2);
Kor = 1.16e-5*exp( Aor/R*( 1/866.7 - 1/Trg ) );
Kod = 6.34e-9*Rai^2;
Ofg = 21*exp( - (Prg*Hrg/Rai) / ( 1/Kod + 1/(Kor*Crc) ) );
Rcb = Rai*(21-Ofg)/242;
sys = [x;Fcf;Rcb;Kor];
% End of mdlOutputs.